==> jspice3-2.5.tar.gz: jspice3-2.5/src/lib/inp/inp2dot.c <==
/***************************************************************************
JSPICE3 adaptation of Spice3f2 - Copyright (c) Stephen R. Whiteley 1992
Copyright 1990 Regents of the University of California.  All rights reserved.
Authors: 1987 Thomas L. Quarles
         1992 Stephen R. Whiteley
****************************************************************************/

#include "spice.h"
#include "ftedefs.h"
#include "inpdefs.h"
#include "inpmacs.h"
#include "jobdefs.h"
#include "misc.h"

#ifdef __STDC__
static int get_anal(char*);
#else
static int get_anal();
#endif

int
INP2dot(ckt,tab,current,task,gnode)

GENERIC *ckt;
INPtables *tab;
card *current;
GENERIC *task;
GENERIC *gnode;
{
    /* .<something> Many possibilities */

    char *line;     /* the part of the current line left to parse */
    char *name;     /* the device's name */
    GENERIC *node;  /* the first node's node pointer */
    IFvalue ptemp;  /* a value structure to package stuff into */
    IFparm *prm;    /* pointer to parameter to search through array */
    char *token;    /* a token from the line */
    int which;      /* which analysis we are performing */
    int length;     /* length of a name */
    int error;      /* error code temporary */


    line = current->line;
    INPgetTok(&line,&token,1);

    if (strcmp(token,".model") == 0) {
        /* don't have to do anything, since models were all done in
         * pass 1 
         */
        txfree(token);
        return (0);
    }

    if ( (strcmp(token,".width") == 0) ||
         (strcmp(token,".print") == 0) ||
         (strcmp(token,".plot" ) == 0) ) {
        /* obsolete - ignore */
        LITERR("Obsolete control card - ignored.")
        txfree(token);
        return (0);
    }

    if (strcmp(token,".temp") == 0) {
        /* .temp temp1 temp2 temp3 temp4 ..... */
        /* not yet implemented - warn & ignore */
        LITERR(".TEMP card obsolete - use .options TEMP and TNOM.")
        txfree(token);
        return (0);
    }

    if ( (strcmp(token,".four") == 0) ||
         (strcmp(token,".fourier") == 0) ) {
        /* .four */
        /* not implemented - warn & ignore */
        LITERR("Use fourier command to obtain fourier analysis.")
        txfree(token);
        return (0);
    }

    if (strcmp(token,".nodeset") == 0) {
        /* .nodeset */
        txfree(token);
        which = -1;
        for (prm = ft_sim->nodeParms;
                prm < ft_sim->nodeParms+ft_sim->numNodeParms; prm++) {
            if (strcmp(prm->keyword,"nodeset") == 0) {
                which = prm->id;
                break;
            }
        }
        if (which == -1) {
            LITERR("Nodeset unknown to simulator.")
            return (0);
        }
        for (;;) { /* loop until we run out of data */
            INPgetTok(&line,&name,1);
            /* check to see if in the form V(xxx) and grab the xxx */
            if (*name == (char)NULL) break; /* end of line */
            length = strlen(name);
            if ((*name == 'V' || *(name) == 'v') && (length == 1)) {
                /* looks like V - must be V(xx) - get xx now*/
                INPgetTok(&line,&name,1);
                INPtermInsert(ckt,&name,tab,&node);
                ptemp.rValue = INPevaluate(&line,&error,1);
                IFC(setNodeParm,(ckt,node,which,&ptemp,(IFvalue*)NULL))
                continue;
            }
            LITERR(".NODESET syntax error.")
            break;
        }
        return (0);
    }

    if (strcmp(token,".ic") == 0) {
        /* .ic */
        txfree(token);
        which = -1;
        for (prm = ft_sim->nodeParms;
                prm < ft_sim->nodeParms+ft_sim->numNodeParms; prm++) {
            if (strcmp(prm->keyword,"ic") == 0) {
                which = prm->id;
                break;
            }
        }
        if (which == -1) {
            LITERR("IC unknown to simulator.")
            return (0);
        }
        for (;;) { /* loop until we run out of data */
            INPgetTok(&line,&name,1);
            /* check to see if in the form V(xxx) and grab the xxx */
            if (*name == (char)NULL) break; /* end of line */
            length = strlen(name);
            if ((*name == 'V' || *(name) == 'v') && (length == 1)) {
                /* looks like V - must be V(xx) - get xx now*/
                INPgetTok(&line,&name,1);
                INPtermInsert(ckt,&name,tab,&node);
                ptemp.rValue = INPevaluate(&line,&error,1);
                IFC(setNodeParm,(ckt,node,which,&ptemp,(IFvalue*)NULL))
                continue;
            }
            LITERR(".IC syntax error.")
            break;
        }
        return (0);
    }

    if (strcmp(token,".op") == 0) {
        /* .op */
        txfree(token);
        which = get_anal("DCOP");
        if (which == -1) {
            LITERR("DC operating point analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".tran") == 0) {
        /* .tran Tstep Tstop <Tstart <Tmax> > <UIC> */
        txfree(token);
        which = get_anal("TRAN");
        if (which == -1) {
            LITERR("Transient analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".dc") == 0) {
        /* .dc SRC1NAME Vstart1 Vstop1 Vinc1 [SRC2NAME Vstart2 */
        /*        Vstop2 Vinc2 */
        txfree(token);
        which = get_anal("DCTransfer");
        if (which == -1) {
            LITERR("DC transfer curve analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".tf") == 0) {
        /* .tf v( node1, node2 ) src */
        /* .tf vsrc2             src */
        txfree(token);
        which = get_anal("TF");
        if (which == -1) {
            LITERR("DC Transfer function analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".ac") == 0) {
        /* .ac {DEC OCT LIN} NP FSTART FSTOP */
        txfree(token);
        which = get_anal("AC");
        if (which == -1) {
            LITERR("AC small signal analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".pz") == 0) {
        /* .pz nodeI nodeG nodeJ nodeK {V I} {POL ZER PZ} */
        txfree(token);
        which = get_anal("PZ");
        if (which == -1) {
            LITERR("Pole-zero analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".disto") == 0) {
        txfree(token);
        /* .disto {DEC OCT LIN} NP FSTART FSTOP <F2OVERF1> */
        which = get_anal("DISTO");
        if (which == -1) {
            LITERR("Small signal distortion analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".noise") == 0) {
        /* .noise V(OUTPUT) SRC {DEC OCT LIN} NP FSTART FSTOP <PTSPRSUM> */
        txfree(token);
        which = get_anal("NOISE");
        if (which == -1) {
            LITERR("Noise analysis unsupported.");
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token, ".sens") == 0) {
        /* Format is:
         *  .sens <output>
         *  + [ac [dec|lin|oct] <pts> <low freq> <high freq> | dc ]
         */
        txfree(token);
        which = get_anal("SENS");
        if (which == -1) {
            LITERR("Sensitivity unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return(0);
    }

    if ((strcmp(token,".options") == 0) ||
        (strcmp(token,".option" ) == 0) ||
        (strcmp(token,".opt"    ) == 0)) {
        /* .option - specify program options */
        txfree(token);
        which = get_anal("options");
        if (which == -1) {
            LITERR("Analysis options table not found.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if ((strcmp(token,".subckt") == 0) ||
        (strcmp(token,".ends"  ) == 0)) {
        /* not yet implemented - warn & ignore */
        LITERR("Subcircuits not yet implemented - ignored.")
        txfree(token);
        return (0);
    }

    if (strcmp(token,".end") == 0) {
        /* .end - end of input */
        /* not allowed to pay attention to additional input - return */
        txfree(token);
        return (1);
        /*NOTREACHED*/
    }

    if (strcmp(token,".table") == 0) {
        txfree(token);
        current->error = INPerrCat(current->error,INPtablCheck(line,ckt));
        return 0;
    }

    LITERR("Unimplemented control card.")
    txfree(token);
    return (0);
}

static int
get_anal(string)
char *string;
{
    int i;

    for (i = 0; i < ft_sim->numAnalyses; i++) {
        if (strcmp(ft_sim->analyses[i]->name,string) == 0) {
            return (i);
        }
    }
    return (-1);
}

==> jspice3-2.5-011109.tar.gz: jspice3-2.5/src/lib/inp/inp2dot.c <==
/***************************************************************************
JSPICE3 adaptation of Spice3f2 - Copyright (c) Stephen R. Whiteley 1992
Copyright 1990 Regents of the University of California.  All rights reserved.
Authors: 1987 Thomas L. Quarles
         1992 Stephen R. Whiteley
****************************************************************************/

#include "spice.h"
#include "ftedefs.h"
#include "inpdefs.h"
#include "inpmacs.h"
#include "jobdefs.h"
#include "misc.h"

#ifdef __STDC__
static int get_anal(char*);
#else
static int get_anal();
#endif

int
INP2dot(ckt,tab,current,task,gnode)

GENERIC *ckt;
INPtables *tab;
card *current;
GENERIC *task;
GENERIC *gnode;
{
    /* .<something> Many possibilities */

    char *line;     /* the part of the current line left to parse */
    char *name;     /* the device's name */
    GENERIC *node;  /* the first node's node pointer */
    IFvalue ptemp;  /* a value structure to package stuff into */
    IFparm *prm;    /* pointer to parameter to search through array */
    char *token;    /* a token from the line */
    int which;      /* which analysis we are performing */
    int length;     /* length of a name */
    int error;      /* error code temporary */


    line = current->line;
    INPgetTok(&line,&token,1);

    if (strcmp(token,".model") == 0) {
        /* don't have to do anything, since models were all done in
         * pass 1 
         */
        txfree(token);
        return (0);
    }

    if ( (strcmp(token,".width") == 0) ||
         (strcmp(token,".print") == 0) ||
         (strcmp(token,".plot" ) == 0) ) {
        /* obsolete - ignore */
        LITERR("Obsolete control card - ignored.")
        txfree(token);
        return (0);
    }

    if (strcmp(token,".temp") == 0) {
        /* .temp temp1 temp2 temp3 temp4 ..... */
        /* not yet implemented - warn & ignore */
        LITERR(".TEMP card obsolete - use .options TEMP and TNOM.")
        txfree(token);
        return (0);
    }

    if ( (strcmp(token,".four") == 0) ||
         (strcmp(token,".fourier") == 0) ) {
        /* .four */
        /* not implemented - warn & ignore */
        LITERR("Use fourier command to obtain fourier analysis.")
        txfree(token);
        return (0);
    }

    if (strcmp(token,".nodeset") == 0) {
        /* .nodeset */
        txfree(token);
        which = -1;
        for (prm = ft_sim->nodeParms;
                prm < ft_sim->nodeParms+ft_sim->numNodeParms; prm++) {
            if (strcmp(prm->keyword,"nodeset") == 0) {
                which = prm->id;
                break;
            }
        }
        if (which == -1) {
            LITERR("Nodeset unknown to simulator.")
            return (0);
        }
        for (;;) { /* loop until we run out of data */
            INPgetTok(&line,&name,1);
            /* check to see if in the form V(xxx) and grab the xxx */
            if (*name == 0) break; /* end of line */
            length = strlen(name);
            if ((*name == 'V' || *(name) == 'v') && (length == 1)) {
                /* looks like V - must be V(xx) - get xx now*/
                INPgetTok(&line,&name,1);
                INPtermInsert(ckt,&name,tab,&node);
                ptemp.rValue = INPevaluate(&line,&error,1);
                IFC(setNodeParm,(ckt,node,which,&ptemp,(IFvalue*)NULL))
                continue;
            }
            LITERR(".NODESET syntax error.")
            break;
        }
        return (0);
    }

    if (strcmp(token,".ic") == 0) {
        /* .ic */
        txfree(token);
        which = -1;
        for (prm = ft_sim->nodeParms;
                prm < ft_sim->nodeParms+ft_sim->numNodeParms; prm++) {
            if (strcmp(prm->keyword,"ic") == 0) {
                which = prm->id;
                break;
            }
        }
        if (which == -1) {
            LITERR("IC unknown to simulator.")
            return (0);
        }
        for (;;) { /* loop until we run out of data */
            INPgetTok(&line,&name,1);
            /* check to see if in the form V(xxx) and grab the xxx */
            if (*name == 0) break; /* end of line */
            length = strlen(name);
            if ((*name == 'V' || *(name) == 'v') && (length == 1)) {
                /* looks like V - must be V(xx) - get xx now*/
                INPgetTok(&line,&name,1);
                INPtermInsert(ckt,&name,tab,&node);
                ptemp.rValue = INPevaluate(&line,&error,1);
                IFC(setNodeParm,(ckt,node,which,&ptemp,(IFvalue*)NULL))
                continue;
            }
            LITERR(".IC syntax error.")
            break;
        }
        return (0);
    }

    if (strcmp(token,".op") == 0) {
        /* .op */
        txfree(token);
        which = get_anal("DCOP");
        if (which == -1) {
            LITERR("DC operating point analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".tran") == 0) {
        /* .tran Tstep Tstop <Tstart <Tmax> > <UIC> */
        txfree(token);
        which = get_anal("TRAN");
        if (which == -1) {
            LITERR("Transient analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".dc") == 0) {
        /* .dc SRC1NAME Vstart1 Vstop1 Vinc1 [SRC2NAME Vstart2 */
        /*        Vstop2 Vinc2 */
        txfree(token);
        which = get_anal("DCTransfer");
        if (which == -1) {
            LITERR("DC transfer curve analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".tf") == 0) {
        /* .tf v( node1, node2 ) src */
        /* .tf vsrc2             src */
        txfree(token);
        which = get_anal("TF");
        if (which == -1) {
            LITERR("DC Transfer function analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".ac") == 0) {
        /* .ac {DEC OCT LIN} NP FSTART FSTOP */
        txfree(token);
        which = get_anal("AC");
        if (which == -1) {
            LITERR("AC small signal analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".pz") == 0) {
        /* .pz nodeI nodeG nodeJ nodeK {V I} {POL ZER PZ} */
        txfree(token);
        which = get_anal("PZ");
        if (which == -1) {
            LITERR("Pole-zero analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".disto") == 0) {
        txfree(token);
        /* .disto {DEC OCT LIN} NP FSTART FSTOP <F2OVERF1> */
        which = get_anal("DISTO");
        if (which == -1) {
            LITERR("Small signal distortion analysis unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token,".noise") == 0) {
        /* .noise V(OUTPUT) SRC {DEC OCT LIN} NP FSTART FSTOP <PTSPRSUM> */
        txfree(token);
        which = get_anal("NOISE");
        if (which == -1) {
            LITERR("Noise analysis unsupported.");
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if (strcmp(token, ".sens") == 0) {
        /* Format is:
         *  .sens <output>
         *  + [ac [dec|lin|oct] <pts> <low freq> <high freq> | dc ]
         */
        txfree(token);
        which = get_anal("SENS");
        if (which == -1) {
            LITERR("Sensitivity unsupported.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return(0);
    }

    if ((strcmp(token,".options") == 0) ||
        (strcmp(token,".option" ) == 0) ||
        (strcmp(token,".opt"    ) == 0)) {
        /* .option - specify program options */
        txfree(token);
        which = get_anal("options");
        if (which == -1) {
            LITERR("Analysis options table not found.")
            return (0);
        }
        (*(analInfo[which]->parse))(ckt,tab,which,current,&line,task,gnode);
        return (0);
    }

    if ((strcmp(token,".subckt") == 0) ||
        (strcmp(token,".ends"  ) == 0)) {
        /* not yet implemented - warn & ignore */
        LITERR("Subcircuits not yet implemented - ignored.")
        txfree(token);
        return (0);
    }

    if (strcmp(token,".end") == 0) {
        /* .end - end of input */
        /* not allowed to pay attention to additional input - return */
        txfree(token);
        return (1);
        /*NOTREACHED*/
    }

    if (strcmp(token,".table") == 0) {
        txfree(token);
        current->error = INPerrCat(current->error,INPtablCheck(line,ckt));
        return 0;
    }

    LITERR("Unimplemented control card.")
    txfree(token);
    return (0);
}

static int
get_anal(string)
char *string;
{
    int i;

    for (i = 0; i < ft_sim->numAnalyses; i++) {
        if (strcmp(ft_sim->analyses[i]->name,string) == 0) {
            return (i);
        }
    }
    return (-1);
}
